#include "stdafx.h"
#include <math.h>

//-----------------------------------------------------------------------------
void CGraph :: SetXScale( double dMin, double dMax ){
	m_dXMin=dMin;
	m_dXMax=dMax;
	if( dMax<=dMin ) 
		return;
	int iStep = int(log10((dMax-dMin)/1.25));
	m_dXLabelSep = pow( 10.0, iStep );
	int iSteps = int((dMax-dMin)/m_dXLabelSep);
	m_dXTickMin = 
		m_dXLabelMin = m_dXLabelSep * ceil( dMin/m_dXLabelSep );
	m_dXTickSep = /*iSteps>5 ? m_dXLabelSep :*/ ( iSteps>2 ? m_dXLabelSep/2.0 : m_dXLabelSep/10.0 );
	}

void CGraph :: SetYScale( double dMin, double dMax ){
	m_dYMin=dMin;
	m_dYMax=dMax;
	if( dMax<=dMin ) 
		return;
	int iStep = int(log10((dMax-dMin)/1.25));
	m_dYLabelSep = pow( 10.0, iStep );
	int iSteps = int((dMax-dMin)/m_dYLabelSep);
	m_dYTickMin = 
		m_dYLabelMin = m_dYLabelSep * ceil( dMin/m_dYLabelSep );
	m_dYTickSep = iSteps>5 ? m_dYLabelSep : ( iSteps>2 ? m_dYLabelSep/2.0 : m_dYLabelSep/10.0 );
	}

void CGraph :: SetXScale( ){ 
	SetXScale( XMin(), XMax() );
	}

void CGraph :: SetYScale( ) { 
	SetYScale( YMin(), YMax() );
	}

void CGraph :: DrawXScale( HDC hdc ){
	char buf[64];
	int iCount=0;
	for( double d=m_dXLabelMin; d<m_dXMax && ++iCount<100; d+=m_dXLabelSep ) {
		sprintf( buf, "%g", d );
		DrawXLabel( hdc, buf, d );
		}
	iCount=0;
	for( double d=m_dXTickMin; d<m_dXMax && ++iCount<100; d+=m_dXTickSep )
		DrawXTick( hdc, d );
	}
	
void CGraph :: DrawYScale( HDC hdc ){
	char buf[64];
	int iCount=0;
	for( double d=m_dYLabelMin; d<m_dYMax && ++iCount<100; d+=m_dYLabelSep ) {
		sprintf( buf, "%g", d );
		DrawYLabel( hdc, buf, d );
		}
	iCount=0;
	for( double d=m_dYTickMin; d<m_dYMax && ++iCount<100; d+=m_dYTickSep )
		DrawYTick( hdc, d );
	}

void CGraph :: SetTitles( const char * pchGraphTitle, const char * pchXTitle, const char * pchYTitle ) { 
	m_cTitle=pchGraphTitle;
	m_cXTitle=pchXTitle;
	m_cYTitle=pchYTitle;
	}
	
void CGraph :: SetRectangles( RECT & rctPic ){
	SetRect( &m_rctPic, rctPic.left, rctPic.top, rctPic.right, rctPic.bottom );
	SetRect( &m_rctData, rctPic.left+MARGIN_LFT+YTEXT_WIDTH+YLABEL_WIDTH, 
					rctPic.top+MARGIN_TOP, 
					rctPic.right-MARGIN_RGT, 
					rctPic.bottom-MARGIN_BTM-XTEXT_HEIGHT-XLABEL_HEIGHT );
	}

void CGraph :: DrawDataFrame( HDC hdc ) { 
	//-- Data frame 		
	MoveToEx( hdc, m_rctData.left, m_rctData.top, NULL );
	LineTo( hdc, m_rctData.left, m_rctData.bottom );
	LineTo( hdc, m_rctData.right, m_rctData.bottom );
	}
		
void CGraph :: DrawXTitle( HDC hdc ){
	//-- X Title
	RECT rct;
	SetRect( &rct, m_rctData.left, m_rctPic.bottom-XTEXT_HEIGHT, m_rctData.right, m_rctPic.bottom );
	DrawText( hdc, m_cXTitle.ConstBuffer(), m_cXTitle.Length(), &rct, DT_CENTER | DT_VCENTER);
	}

void CGraph :: DrawYTitle( HDC hdc ){
	RECT rct;
	//-- Y Title - prepare for vertical printing
	HFONT fnt = CreateFont( -12, 0, 900, 900, FW_REGULAR, 0, 0, 0, 
						DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
						PROOF_QUALITY /*DEFAULT_QUALITY*/, 
						FF_ROMAN /*FF_SWISS */ | VARIABLE_PITCH, NULL );
	HFONT fntWas = (HFONT) SelectObject( hdc, fnt );
	UINT uiAlignWas = SetTextAlign( hdc, TA_CENTER );
	SetRect( &rct, m_rctPic.left+MARGIN_LFT, 
					m_rctPic.top+MARGIN_TOP, 
					m_rctPic.left+MARGIN_LFT+YTEXT_WIDTH, 
					m_rctData.bottom );

	//-- send text				//DrawText( hdc, m_cYTitle.ConstBuffer(), m_cYTitle.Length(), &rct, 0 );
	ExtTextOut( hdc, rct.left, (rct.bottom+rct.top)/2, ETO_CLIPPED, &rct, 
				m_cYTitle.ConstBuffer(), m_cYTitle.Length(), NULL);
	
	//-- restore to normal printing 
	if( uiAlignWas != GDI_ERROR ) 	
		SetTextAlign( hdc, uiAlignWas);
	SelectObject( hdc, fntWas );
	DeleteObject( fnt );
	}
	
void CGraph :: DrawXLabel( HDC hdc, const char * pchLabel, double d ) { 
	double dRelPos = (d-m_dXMin)/(m_dXMax-m_dXMin);
	int iXPos=int(dRelPos*m_rctData.right+(1.0-dRelPos)*m_rctData.left);

	//-- Text Size 
	int iLen = strlen(pchLabel);
	SIZE pt;
	GetTextExtentPoint32( hdc, pchLabel, iLen, &pt );
	
	//-- Text Draw
	RECT rct;
	SetRect( &rct, iXPos-pt.cx/2-1, m_rctData.bottom+1, iXPos+pt.cx/2+1, m_rctData.bottom+XLABEL_HEIGHT );
	DrawText( hdc, pchLabel, iLen, &rct, DT_CENTER | DT_VCENTER );
	}

void CGraph :: DrawXTick( HDC hdc, double d ) { 
	double dRelPos = (d-m_dXMin)/(m_dXMax-m_dXMin);
	int iXPos=int(dRelPos*m_rctData.right+(1.0-dRelPos)*m_rctData.left);
	//-- Tick mark
	MoveToEx( hdc, iXPos, m_rctData.bottom, NULL );
	LineTo( hdc, iXPos, m_rctData.bottom+X_TICKHEIGHT );
	}

void CGraph :: DrawYLabel( HDC hdc, const char * pchLabel, double d ) { 
	double dRelPos = (d-m_dYMin)/(m_dYMax-m_dYMin);
	int iYPos=int(dRelPos*m_rctData.top+(1.0-dRelPos)*m_rctData.bottom);
	//-- Text 
	int iLen = strlen(pchLabel);
	RECT rct;
	SetRect( &rct, m_rctData.left-YLABEL_WIDTH, iYPos-YLABEL_HEIGHT, m_rctData.left-1, iYPos );
	DrawText( hdc, pchLabel, iLen, &rct, DT_RIGHT | DT_VCENTER );
	}

void CGraph :: DrawYTick( HDC hdc, double d ) { 
	double dRelPos = (d-m_dYMin)/(m_dYMax-m_dYMin);
	int iYPos=int(dRelPos*m_rctData.top+(1.0-dRelPos)*m_rctData.bottom);
	//-- Tick mark
	MoveToEx( hdc, m_rctData.left-Y_TICKWIDTH, iYPos, NULL );
	LineTo( hdc, m_rctData.left, iYPos );
	}

//-----------------------------------------------------------------------------
CHistogram :: CHistogram( int iItems, double * aData, int iBins ) { 
	m_iType=HISTOGRAM;
	if( (m_iItems=iItems)<=0 ) 
		return;

	//-- Find min, max, binwidth
	double dMax=aData[0];
	m_dMin=aData[0];
	
	for( int i=1; i<iItems; ++i ) {
		dMax = max( dMax, aData[i] );
		m_dMin = min( m_dMin, aData[i] );
		}
		
	m_dBinWidth = (dMax-m_dMin)/iBins;

	/*
	Note : There is a very minor error here related to the fact that the last bin should
	contain values  dMin+(iBins-1)*dBinWidth <= d < dMax, but exactly dMax exists in the 
	dataset and is included in the last bin.  Not particularly relevant though.
	*/

	//-- create bins
	if( Create( iBins, m_dMin, m_dBinWidth ) ) {
		AddData( iItems, aData );
		SetXScale();
		SetYScale();
		}
		
	}

CHistogram :: CHistogram( int iBins, double dMin, double dBinWidth ) { 
	m_iType=HISTOGRAM;
	Create( iBins, dMin, dBinWidth );
	m_crFill = RGB(0,0,255);
	m_crLine = RGB(0,0,0);
	m_crCurve= RGB(255,0,0);
	}

CHistogram :: ~CHistogram() { 
	if( m_aiBin != NULL )  
		delete [] m_aiBin;
	}

bool CHistogram :: Create( int iBins, double dMin, double dBinWidth ) { 
	m_iItems=0;
	if( (m_iBins=iBins)<=0 || (m_aiBin=new int[iBins])==NULL ) { 
		m_iBins=0;
		m_aiBin=NULL;
		return false;
		}
	for( int i=0; i<iBins; ++i )
		m_aiBin[i]=0;
	m_dMin=dMin;
	m_dBinWidth=dBinWidth;
	return true;
	}

void CHistogram :: AddData( int iItems, double * aData ) { 
	for( int i=0; i<iItems; ++i ) 
		++m_aiBin[max(0,min(m_iBins-1,int((aData[i]-m_dMin)/m_dBinWidth)))];		
	m_iItems+=iItems;
	}

void CHistogram :: AddDatum( double d ) { 
	++m_aiBin[max(0,min(m_iBins-1,int((d-m_dMin)/m_dBinWidth)))];		
	++m_iItems;
	}
	
void CHistogram :: Draw( HDC hdc, RECT rctPic ) { 
	SetRectangles( rctPic );
	DrawDataFrame( hdc );

	DrawXTitle( hdc );
	DrawXScale( hdc );

	DrawYTitle( hdc );
	DrawYScale( hdc );
	
	if( m_iBins <= 0 ) 
		return;
		
	//-- Bars 
	int iMax = 0;
	for( int iBin=0; iBin<m_iBins; ++iBin ) {
		if( m_aiBin[iBin]>iMax ) 
			iMax=m_aiBin[iBin];
		}
			
	HPEN penLine = CreatePen( 0, 0, m_crLine );
	HPEN penWas = (HPEN) SelectObject( hdc, penLine );
	HBRUSH brFill = CreateSolidBrush( m_crFill );
	HBRUSH brWas = (HBRUSH) SelectObject( hdc, brFill );
	
	for( int iBin=0; iBin<m_iBins; ++iBin ) { 
		int iHeight = MulDiv( m_aiBin[iBin], m_rctData.bottom-m_rctData.top, iMax );
		int iX = m_rctData.left+MulDiv(iBin,(m_rctData.right-m_rctData.left),m_iBins);
		Rectangle( hdc, m_rctData.left+MulDiv(iBin,(m_rctData.right-m_rctData.left),m_iBins), 
						m_rctData.bottom, 
						m_rctData.left+MulDiv(iBin+1,(m_rctData.right-m_rctData.left),m_iBins)-1, 
						m_rctData.bottom-iHeight );
		MoveToEx( hdc, iX, m_rctData.bottom, NULL );
		LineTo( hdc, iX, m_rctData.bottom-iHeight );
		}
	SelectObject( hdc, brWas );
	SelectObject( hdc, penWas );
	DeleteObject( penLine );
	DeleteObject( brFill );	
	
	}
	
double CHistogram :: XMin() {
	return m_dMin;
	}
	
double CHistogram :: XMax() { 
	return m_dMin + m_dBinWidth*(m_iBins+1);
	}

double CHistogram :: YMin() {
	return 0.0;
	}
	
double CHistogram :: YMax() { 
	int iMax=0;
	for( int i=0; i<m_iBins; ++i ) 
		iMax=max( iMax, m_aiBin[i] );
	return double(iMax);
	}
	
	